@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 import json
+from multiprocessing import RLock
 import threading
 import time
 
@@ -32,6 +33,13 @@ def setMqttLog(PrintLog=False):
 class Dxr_url_port:
     pass
 
+def get_cond(topic):
+    global callback_dit
+    if not  isinstance(topic, str):
+        topic = topic.topic
+    if topic not in callback_dit:
+        Dxr_Subscriber(topic, None)
+    return callback_dit[topic]['cond']
 
 def on_connect(client, userdata, flags, rrc):
     global rc
@@ -44,15 +52,14 @@ def on_message(client, obj, msg):
     global callback_dit
     topic = msg.topic
     try:
-        threading.Thread(target=callback, args=(topic, msg, client), daemon=True).start()
+        # threading.Thread(target=callback, args=(topic, msg, client), daemon=True).start()
+        callback(topic, msg, client)
     except Exception as ex:
         print(ex)
         keys = callback_dit.keys()
         topic_arr = topic.split("/")[1:]
-        print(topic_arr)
         for item in keys:
             item_arr = item.split("/")[1:]
-            print(item_arr)
             isThisTopic = True
             if len(item_arr) == len(topic_arr):
                 for i in range(len(item_arr)):
@@ -62,16 +69,24 @@ def on_message(client, obj, msg):
                         isThisTopic = False
                         break
             if isThisTopic:
-                threading.Thread(target=callback, args=(item, msg, client), daemon=True).start()
+                # threading.Thread(target=callback, args=(item, msg, client), daemon=True).start()
+                callback(item, msg, client)
                 break
 
 
 def callback(topic, msg, client):
+    global callback_dit
     msg = str(msg.payload.decode("utf-8"))
     while type(msg) is str:
         msg = json.loads(msg)
-    callback_dit[topic](msg, topic)
-    pass
+    if topic not in callback_dit:
+        callback_dit[topic] = {'func': None, 'cond': threading.Event(), 'msg': None}
+    callback_dit[topic]['msg'] = msg
+    cond = get_cond(topic)
+    cond.set()
+    cond.clear()
+    if callback_dit[topic]['func'] is not None:
+        callback_dit[topic]['func'](msg, topic)
 
 
 # 一旦订阅成功，回调此方法
@@ -101,7 +116,7 @@ def Mqtt():
             # 连接broker，心跳时间为60s
             mqttc.connect(server_url, server_port, 60)
             # 订阅该主题，QoS=0
-            t = threading.Thread(target=mqttc.loop_forever)
+            t = threading.Thread(target=mqttc.loop_forever, daemon=True)
             t.start()
             return mqttc
         except Exception as ex:
@@ -122,7 +137,11 @@ class Dxr_Subscriber:
             self.mqttc.subscribe(topic.topic)
             self.topic = topic.topic
         self.callback = callback
-        callback_dit[self.topic] = callback
+        if self.topic not in callback_dit:
+            cond = threading.Event()
+            callback_dit[self.topic] = {'func': self.callback, 'cond': cond, 'msg': None}
+        else:
+            callback_dit[self.topic]['func'] = self.callback
         topic_list_str_sub = client_id + '_sub:\n'
         for item in callback_dit.keys():
             topic_list_str_sub = '' + topic_list_str_sub + item + '\n'
@@ -148,7 +167,7 @@ class dxr_subscriber:
 
 class Dxr_Publisher:
     def __init__(self, topic):
-        global publish_dit, topic_list_str_pub, topic_list_str_sub
+        global publish_dit, topic_list_str_pub, topic_list_str_sub, callback_dit
         if isinstance(topic, str):
             self.topic = topic
             self.data_type = None
@@ -169,6 +188,42 @@ class Dxr_Publisher:
             self.mqttc.publish(self.topic, msg.get_json(), qos=0)
         else:
             self.mqttc.publish(self.topic, json.dumps(msg), qos=0)
+    
+    '''
+    使用await_publish来实现消息闭环
+    await_publish(msg, timeout, topic)
+    {
+        msg 为发送的消息
+        timeout 不指定的时候，为一直等待设定话题的消息，指定后超时时间中未收到消息，会接收到一个None消息
+        topic 为指定闭环消息的话题类型，如果不指定，则默认为原话题类型后追加'_response',可以传递字符串类型的话题，也可以传递消息类型
+    }
+    '''      
+    def await_publish(self, msg, timeout=None, topic=None):
+        global callback_dit
+        if self.data_type:
+            self.mqttc.publish(self.topic, msg.get_json(), qos=0)
+        else:
+            self.mqttc.publish(self.topic, json.dumps(msg), qos=0)
+        await_topic = None
+        if topic is None: 
+            await_topic = self.topic + '_response'
+        else:
+            if isinstance(topic, str):
+                await_topic = topic
+            else:
+                await_topic = topic.topic
+        if await_topic is None:
+            return None
+        print('await_publish: ' + await_topic)
+        cond = get_cond(await_topic)
+        if cond is None:
+            return None
+        if timeout is None:
+            cond.wait()
+        else:
+            cond.wait(timeout)
+        return callback_dit[await_topic]['msg']
+        
 
 
 class Dxr_UnSubscriber:
