@@ -1226,3 +1226,53 @@ class actuator_ptz_control(msg):
     def __init__(self):
         super().__init__()
         self.msg = self.msg_class()  # 消息体
+        
+#pc端开启关闭话筒
+class audio_voice_control(msg):
+    """
+    定义一个应用消息,消息结构如下：
+    {
+        "msg":{
+            "sn":"123",//web端发送带sn,服务端转发给客户端不需要sn
+            "action":1 ,//1:话筒开;0:话筒关,int类型
+        }
+    }
+    """
+    topic = "/audio/voice/control"
+    
+    class msg_class(NestedClass):
+        def __init__(self):
+            super().__init__()
+            self.sn = ""  # String型
+            self.action = ""  # Int型
+            
+    def __init__(self):
+        super().__init__()
+        self.msg = self.msg_class()  # 消息体
+        
+#pc端开启关闭话筒返回值 
+class audio_voice_control_response(msg):
+    """
+    定义一个应用消息,消息结构如下：
+    {
+        "msg":{
+            "error_code":0,
+            "error_msg":"",
+            "action:0, //结果,0:关闭,1:开启
+            "result":1 //0:失败,1:成功
+        }
+    }
+    """
+    topic = "/audio/voice/control_response"
+
+    class msg_class(NestedClass):
+        def __init__(self):
+            super().__init__()
+            self.error_code = ""  # int型
+            self.error_msg = ""  # string型
+            self.action = ""  # int型
+            self.result = ""  # int型
+
+    def __init__(self):
+        super().__init__()
+        self.msg = self.msg_class()  # 消息体
